//////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RSContext.h"
#include "RSMenu.h"

#include "Registry.H"

//////////////////////////////////////////////////////////////////////////////////////////

#define PriorityKey "Software\\NetWorkDLS\\448Delta\\Priority"
#define EncDlgKey   "Software\\NetWorkDLS\\448Delta\\EncryptDialog"
#define DecDlgKey   "Software\\NetWorkDLS\\448Delta\\DecryptDialog"
#define StatDlgKey  "Software\\NetWorkDLS\\448Delta\\StatusDialog"
#define PathKey     "Software\\NetWorkDLS\\448Delta\\Path"

//////////////////////////////////////////////////////////////////////////////////////////

HRESULT CRSMenu::QueryContextMenu(HMENU hmenu, UINT indexMenu, UINT idCmdFirst, UINT idCmdLast, UINT uFlags)
{
    if (!(CMF_DEFAULTONLY & uFlags))
    {
        int OpenWith448Delta   = idCmdFirst;
        HBITMAP BitMapResource = NULL;

        char MenuIcon[1024];
        unsigned long SizeOfStringBuffer = 1024;

        Get_StringRegistryValue(HKEY_LOCAL_MACHINE, PathKey, "MenuIcon", MenuIcon, SizeOfStringBuffer);

        indexMenu++;
        InsertMenu(hmenu, indexMenu, MF_STRING|MF_BYPOSITION, OpenWith448Delta, _T("&448Delta Encryption"));
        BitMapResource = (HBITMAP) LoadImage(0,MenuIcon,IMAGE_BITMAP, 0, 0,LR_LOADFROMFILE | LR_CREATEDIBSECTION);
        SetMenuItemBitmaps(hmenu, indexMenu, MF_BYPOSITION, BitMapResource, BitMapResource);

        // A successful return code should be the offset of
        // the largest command identifier assigned, plus one
        return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, OpenWith448Delta - idCmdFirst + 1);
    }

    return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, USHORT(0));
}

//////////////////////////////////////////////////////////////////////////////////////////

HRESULT CRSMenu::InvokeCommand(LPCMINVOKECOMMANDINFO pici)
{
    // The offset will determine which option was requested:
    UINT idOffset = LOWORD(pici->lpVerb);

    char StringBuffer[1024];
    unsigned long SizeOfStringBuffer = 1024;

    int LocalLoop = 0;
    while(LocalLoop < 1024)
    {
        StringBuffer[LocalLoop] = '\0';
        LocalLoop++;
    }

    if(idOffset == 0)
    {
        if(!Get_StringRegistryValue(HKEY_LOCAL_MACHINE, PathKey, "448Delta", StringBuffer, SizeOfStringBuffer))
        {
            MessageBox(NULL, "448Delta Executable Path Key Not Found. \r\nPlease Reinstall 448Delta." ,"Execute Error",0 + MB_ICONHAND + MB_SYSTEMMODAL);
            return S_OK;
        }
    }

    HANDLE Result = ShellExecute(pici->hwnd, NULL, _T(StringBuffer), m_szFile, NULL, 0);

    if(Result == (VOID *) 0)                      MessageBox(NULL, "The operating system is out of memory or resources."                                               ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) ERROR_FILE_NOT_FOUND)   MessageBox(NULL, "The specified file was not found."                                                                 ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) ERROR_PATH_NOT_FOUND)   MessageBox(NULL, "The specified path was not found."                                                                 ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) ERROR_BAD_FORMAT)       MessageBox(NULL, "The .EXE file is invalid (non-Win32 .EXE or error in .EXE image)."                                 ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_ACCESSDENIED)    MessageBox(NULL, "The operating system denied access to the specified file."                                         ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_ASSOCINCOMPLETE) MessageBox(NULL, "The filename association is incomplete or invalid."                                                ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_DDEBUSY)         MessageBox(NULL, " The DDE transaction could not be completed because other DDE transactions were being processed."  ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_DDEFAIL)         MessageBox(NULL, "The DDE transaction failed."                                                                       ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_DDETIMEOUT)      MessageBox(NULL, "The DDE transaction could not be completed because the request timed out."                         ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_DLLNOTFOUND)     MessageBox(NULL, "The specified dynamic-link library was not found."                                                 ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_NOASSOC)         MessageBox(NULL, "There is no application associated with the given filename extension."                             ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_OOM)             MessageBox(NULL, "There was not enough memory to complete the operation."                                            ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_PNF)             MessageBox(NULL, "The specified path was not found."                                                                 ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);
    if(Result == (VOID *) SE_ERR_SHARE)           MessageBox(NULL, "A sharing violation occurred."                                                                     ,"Execute Error", MB_ICONHAND + MB_SYSTEMMODAL);


    return S_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////

HRESULT CRSMenu::GetCommandString(UINT idCmd, UINT uFlags, UINT* pwReserved,
                                   LPSTR pszName, UINT cchMax)
{
    if (uFlags & GCS_HELPTEXT)
    {
        // Windows NT will ask for Unicode strings
        switch(idCmd)
        {
        case 0:
            if (uFlags & GCS_VERBA)
                lstrcpyn(pszName, "Open this file with 448Delta Encryption.", cchMax);
            else
                lstrcpynW((LPWSTR)pszName, L"Open this file with 448Delta Encryption.", cchMax);
            break;
        /*
        case 1:
            if (uFlags & GCS_VERBA)
                lstrcpyn(pszName, "Create a Self-Extracting EXe with this file.", cchMax);
            else
                lstrcpynW((LPWSTR)pszName, L"Create a Self-Extracting EXe with this file.", cchMax);
            break;
        */
        default:
            break;
        }

        return S_OK;
    }

    return E_FAIL;
}

//////////////////////////////////////////////////////////////////////////////////////////

HRESULT CRSMenu::Initialize(LPCITEMIDLIST pidlFolder, LPDATAOBJECT lpdobj, HKEY hkeyProgID)
{
    if (lpdobj == NULL)
        return E_INVALIDARG;

    // The file can be referenced through the STGMEDIUM structure
    // that is retrievable through the IDataObject pointer, and
    // DragQueryFile can then be used to retrieve the file name.
    STGMEDIUM medium;
    FORMATETC fe = {CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL};
    HRESULT hr = lpdobj->GetData(&fe, &medium);
    if (FAILED(hr))
        return E_INVALIDARG;

    DragQueryFile(reinterpret_cast<HDROP>(medium.hGlobal), 0,
                    m_szFile, MAX_PATH);

    ReleaseStgMedium(&medium);
    return hr;
}

//////////////////////////////////////////////////////////////////////////////////////////

